@@ -91,16 +91,13 @@ EXCLUDE = (
     "cmd/tools/gvgen.c",
     "cmd/tools/gvpack.cpp",
     "cmd/tools/gxl2gv.c",
-    "cmd/tools/matrix_market.c",
     "cmd/tools/matrix_market.h",
     "cmd/tools/mm2gv.c",
     "cmd/tools/mmio.c",
     "cmd/tools/mmio.h",
-    "cmd/tools/nop.c",
     "cmd/tools/sccmap.c",
     "cmd/tools/tred.c",
     "cmd/tools/unflatten.c",
-    "contrib/diffimg/diffimg.c",
     "contrib/prune/prune.c",
     "doc/libgraph/agmemread.c",
     "doc/libgraph/sccmap.c",
@@ -123,11 +120,9 @@ EXCLUDE = (
     "lib/cdt/dtrenew.c",
     "lib/cdt/dtrestore.c",
     "lib/cdt/dtsize.c",
-    "lib/cdt/dtstat.c",
     "lib/cdt/dtstrhash.c",
     "lib/cdt/dttree.c",
     "lib/cdt/dtview.c",
-    "lib/cdt/dtwalk.c",
     "lib/cgraph/apply.c",
     "lib/cgraph/attr.c",
     "lib/cgraph/cghdr.h",
@@ -168,7 +163,6 @@ EXCLUDE = (
     "lib/common/colorprocs.h",
     "lib/common/colxlate.c",
     "lib/common/const.h",
-    "lib/common/ellipse.c",
     "lib/common/emit.c",
     "lib/common/geom.c",
     "lib/common/geom.h",
@@ -196,12 +190,10 @@ EXCLUDE = (
     "lib/common/taper.c",
     "lib/common/textspan.c",
     "lib/common/textspan.h",
-    "lib/common/timing.c",
     "lib/common/types.h",
     "lib/common/usershape.h",
     "lib/common/utils.c",
     "lib/dotgen/acyclic.c",
-    "lib/dotgen/aspect.c",
     "lib/dotgen/class1.c",
     "lib/dotgen/class2.c",
     "lib/dotgen/cluster.c",
@@ -251,7 +243,6 @@ EXCLUDE = (
     "lib/fdpgen/grid.c",
     "lib/fdpgen/grid.h",
     "lib/fdpgen/layout.c",
-    "lib/fdpgen/tlayout.c",
     "lib/fdpgen/tlayout.h",
     "lib/fdpgen/xlayout.c",
     "lib/fdpgen/xlayout.h",
@@ -325,8 +316,6 @@ EXCLUDE = (
     "lib/mingle/nearest_neighbor_graph_ann.cpp",
     "lib/neatogen/adjust.c",
     "lib/neatogen/adjust.h",
-    "lib/neatogen/bfs.c",
-    "lib/neatogen/bfs.h",
     "lib/neatogen/call_tri.c",
     "lib/neatogen/call_tri.h",
     "lib/neatogen/circuit.c",
@@ -430,7 +419,6 @@ EXCLUDE = (
     "lib/sfdpgen/sparse_solve.c",
     "lib/sfdpgen/spring_electrical.c",
     "lib/sfdpgen/spring_electrical.h",
-    "lib/sfdpgen/stress_model.c",
     "lib/sfio/Sfio_f/_sfslen.c",
     "lib/sfio/sfcvt.c",
     "lib/sfio/sfextern.c",
@@ -533,12 +521,13 @@ EXCLUDE = (
     "plugin/gs/gvloadimage_gs.c",
     "plugin/lasi/gvrender_lasi.cpp",
     "plugin/pango/gvgetfontlist.h",
-    "plugin/pango/gvgetfontlist_pango.c",
     "plugin/pango/gvloadimage_pango.c",
     "plugin/pango/gvplugin_pango.c",
     "plugin/pango/gvrender_pango.c",
     "plugin/pango/gvtextlayout_pango.c",
     "plugin/poppler/gvloadimage_poppler.c",
+    "plugin/quartz/GVTextLayout.h",
+    "plugin/quartz/GVTextLayout.m",
     "plugin/quartz/gvdevice_quartz.c",
     "plugin/quartz/gvloadimage_quartz.c",
     "plugin/quartz/gvplugin_quartz.c",
@@ -566,7 +555,6 @@ EXCLUDE = (
     "tclpkg/tclpathplan/makecw.c",
     "tclpkg/tclpathplan/simple.h",
     "tclpkg/tclpathplan/tclpathplan.c",
-    "tclpkg/tclpathplan/wrapper.c",
     "tclpkg/tclstubs/tclStubLib.c",
     "windows/include/unistd/unistd.h",
 )
