@@ -8,10 +8,13 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include "config.h"
+
 #define STANDALONE
 #include <assert.h>
 #include <sparse/SparseMatrix.h>
 #include <sparse/general.h>
+#include <limits.h>
 #include <math.h>
 #include <sparse/QuadTree.h>
 #include <stdbool.h>
@@ -122,7 +125,7 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
   int i, j, *ia, *ja, u, v;
   SparseMatrix point_poly_map, D;
   double dist;
-  int nbad = 0, flag;
+  int nbad = 0;
   int maxit = 10;
 
   D = SparseMatrix_get_real_adjacency_matrix_symmetrized(graph);
@@ -152,7 +155,7 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
   }
 
   GV_INFO("ratio (edges among discontiguous regions vs total edges)=%f", (double)nbad / ia[n]);
-  stress_model(dim, D, &x, maxit, &flag);
+  const int flag = stress_model(dim, D, x, maxit);
 
   assert(!flag);
 
@@ -247,31 +250,31 @@ static void plot_dot_labels(FILE *f, int n, int dim, double *x, char **labels, f
 
 }
 
-DEFINE_LIST(doubles, double)
+typedef LIST(double) doubles_t;
 
 static void dot_polygon(agxbuf *sbuff, doubles_t xp, doubles_t yp,
                         double line_width, bool fill, const char *cstring) {
 
-  assert(doubles_size(&xp) == doubles_size(&yp));
-  if (!doubles_is_empty(&xp)){
+  assert(LIST_SIZE(&xp) == LIST_SIZE(&yp));
+  if (!LIST_IS_EMPTY(&xp)){
     if (fill) {
       agxbprint(sbuff,
                 " c %" PRISIZE_T " -%s C %" PRISIZE_T " -%s P %" PRISIZE_T " ",
                 strlen(cstring), cstring, strlen(cstring), cstring,
-                doubles_size(&xp));
+                LIST_SIZE(&xp));
     } else {
       if (line_width > 0){
 	size_t len_swidth = (size_t)snprintf(NULL, 0, "%f", line_width);
 	agxbprint(sbuff, " c %" PRISIZE_T " -%s S %" PRISIZE_T
 	          " -setlinewidth(%f) L %" PRISIZE_T " ", strlen(cstring), cstring,
-	          len_swidth + 14, line_width, doubles_size(&xp));
+	          len_swidth + 14, line_width, LIST_SIZE(&xp));
       } else {
 	agxbprint(sbuff, " c %" PRISIZE_T " -%s L %" PRISIZE_T " ", strlen(cstring),
-	          cstring, doubles_size(&xp));
+	          cstring, LIST_SIZE(&xp));
       }
     }
-    for (size_t i = 0; i < doubles_size(&xp); i++) {
-      agxbprint(sbuff, " %f %f", doubles_get(&xp, i), doubles_get(&yp, i));
+    for (size_t i = 0; i < LIST_SIZE(&xp); i++) {
+      agxbprint(sbuff, " %f %f", LIST_GET(&xp, i), LIST_GET(&yp, i));
     }
   }
 }
@@ -285,8 +288,7 @@ static void plot_dot_polygons(agxbuf *sbuff, double line_width,
   const bool use_line = line_width >= 0;
   
   agxbuf cstring_buffer = {0};
-  agxbput(&cstring_buffer, "#aaaaaaff");
-  char *cstring = agxbuse(&cstring_buffer);
+  const char *cstring = "#aaaaaaff";
 
   doubles_t xp = {0};
   doubles_t yp = {0};
@@ -306,11 +308,11 @@ static void plot_dot_polygons(agxbuf *sbuff, double line_width,
 	}
 	dot_polygon(sbuff, xp, yp, line_width, fill, cstring);
 	// start a new polygon
-	doubles_clear(&xp);
-	doubles_clear(&yp);
+	LIST_CLEAR(&xp);
+	LIST_CLEAR(&yp);
       } 
-      doubles_append(&xp, x_poly[2 * ja[j]]);
-      doubles_append(&yp, x_poly[2 * ja[j] + 1]);
+      LIST_APPEND(&xp, x_poly[2 * ja[j]]);
+      LIST_APPEND(&yp, x_poly[2 * ja[j] + 1]);
     }
     if (use_line) {
       dot_polygon(sbuff, xp, yp, line_width, fill, line_color);
@@ -320,8 +322,8 @@ static void plot_dot_polygons(agxbuf *sbuff, double line_width,
     }
   }
   agxbfree(&cstring_buffer);
-  doubles_free(&xp);
-  doubles_free(&yp);
+  LIST_FREE(&xp);
+  LIST_FREE(&yp);
 }
 
 void plot_dot_map(Agraph_t* gr, int n, int dim, double *x, SparseMatrix polys,
@@ -485,12 +487,13 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
   int *comps_ptr = SparseMatrix_weakly_connected_components(BB, &ncomps, &comps);
   SparseMatrix_delete(B);
   SparseMatrix_delete(BB);
-  *poly_point_map = SparseMatrix_new(ncomps, n, n, MATRIX_TYPE_PATTERN, FORMAT_CSR);
+  *poly_point_map = SparseMatrix_new(ncomps, n, (size_t)n, MATRIX_TYPE_PATTERN,
+                                     FORMAT_CSR);
   free((*poly_point_map)->ia);
   free((*poly_point_map)->ja);
   (*poly_point_map)->ia = comps_ptr;
   (*poly_point_map)->ja = comps;
-  (*poly_point_map)->nz = n;
+  (*poly_point_map)->nz = (size_t)n;
 
 }
 
@@ -648,8 +651,7 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps,
   int DEBUG_CYCLE = 0;
   SparseMatrix B;
 
-  ne = E->nz;
-  edge_table = gv_calloc(ne * 2, sizeof(int));
+  edge_table = gv_calloc(E->nz * 2, sizeof(int));
 
   half_edges = SparseMatrix_new(n, n, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
 
@@ -678,7 +680,7 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps,
       }
     }
   }
-  assert(E->nz >= ne);
+  assert(E->nz >= (size_t)ne);
 
   cycle = gv_calloc(ne * 2, sizeof(int));
   B = SparseMatrix_from_coordinate_format_not_compacted(half_edges);
@@ -1004,7 +1006,8 @@ static int make_map_internal(bool include_OK_points, int n, int dim, double *x0,
     int k, t, np=nedgep;
     if (graph && np){
       fprintf(stderr,"add art np = %d\n",np);
-      nz = graph->nz;
+      assert(graph->nz <= INT_MAX);
+      nz = (int)graph->nz;
       y = gv_calloc(dim * n + dim * nz * np, sizeof(double));
       for (i = 0; i < n*dim; i++) y[i] = x[i];
       grouping = gv_calloc(n + nz * np, sizeof(int));
@@ -1291,17 +1294,16 @@ int make_map_from_rectangle_groups(bool include_OK_points,
      
   */
   double *X;
-  int N, nmax, i, j, k, igrp;
-  int *groups, K = *nart;/* average number of points added per side of rectangle */
+  int N, nmax, i, j, igrp;
+  int *groups;
+  double K = *nart; // average number of points added per side of rectangle
 
   double avgsize[2],  avgsz, h[2], p1, p0;
   double point[2];
-  int nadded[2];
-  double delta[2];
   double bbox[4];
 
   if (K < 0){
-    K = (int) 10/(1+n/400.);/* 0 if n > 3600*/
+    K = round(10 / (1 + n / 400.0)); // 0 if n > 3600
   }
   *nart = 0;
   if (Verbose){
@@ -1353,23 +1355,20 @@ int make_map_from_rectangle_groups(bool include_OK_points,
       const double area = (bbox[1] - bbox[0]) * (bbox[3] - bbox[2]);
       shore_depth_tol = sqrt(area / n);
       GV_INFO("setting shore length ======%f", shore_depth_tol);
-    } else {
     }
 
     /* add artificial points in an anti-clockwise fashion */
 
+    double delta[2] = {0};
     if (K > 0){
       delta[0] = .5*avgsize[0]/K; delta[1] = .5*avgsize[1]/K;/* small perturbation to make boundary between labels looks more fractal */
-    } else {
-      delta[0] = delta[1] = 0.;
     }
     for (i = 0; i < n; i++){
       igrp = grouping[i];
+      double nadded[2] = {0};
       for (j = 0; j < 2; j++) {
-	if (avgsz == 0){
-	  nadded[j] = 0;
-	} else {
-	  nadded[j] = (int) K*sizes[i*dim+j]/avgsz;
+	if (avgsz > 0){
+	  nadded[j] = round(K * sizes[i * dim + j] / avgsz);
 	}
       }
 
@@ -1379,7 +1378,7 @@ int make_map_from_rectangle_groups(bool include_OK_points,
 	point[0] = x[i*dim] - sizes[i*dim]/2;
 	p1 = point[1] = x[i*dim+1] + sizes[i*dim + 1]/2;
 	add_point(&N, igrp, &X, &nmax, point, &groups);
-	for (k = 0; k < nadded[0] - 1; k++){
+	for (double k = 0; k < nadded[0] - 1; k++){
 	  point[0] += h[0];
 	  point[1] = p1 + (0.5-drand())*delta[1];
 	  add_point(&N, igrp, &X, &nmax, point, &groups);
@@ -1389,7 +1388,7 @@ int make_map_from_rectangle_groups(bool include_OK_points,
 	point[0] = x[i*dim] + sizes[i*dim]/2;
 	p1 = point[1] = x[i*dim+1] - sizes[i*dim + 1]/2;
 	add_point(&N, igrp, &X, &nmax, point, &groups);
-	for (k = 0; k < nadded[0] - 1; k++){
+	for (double k = 0; k < nadded[0] - 1; k++){
 	  point[0] -= h[0];
 	  point[1] = p1 + (0.5-drand())*delta[1];
 	  add_point(&N, igrp, &X, &nmax, point, &groups);
@@ -1402,7 +1401,7 @@ int make_map_from_rectangle_groups(bool include_OK_points,
 	p0 = point[0] = x[i*dim] - sizes[i*dim]/2;
 	point[1] = x[i*dim+1] - sizes[i*dim + 1]/2;
 	add_point(&N, igrp, &X, &nmax, point, &groups);
-	for (k = 0; k < nadded[1] - 1; k++){
+	for (double k = 0; k < nadded[1] - 1; k++){
 	  point[0] = p0 + (0.5-drand())*delta[0];
 	  point[1] += h[1];
 	  add_point(&N, igrp, &X, &nmax, point, &groups);
@@ -1412,7 +1411,7 @@ int make_map_from_rectangle_groups(bool include_OK_points,
 	p0 = point[0] = x[i*dim] + sizes[i*dim]/2;
 	point[1] = x[i*dim+1] + sizes[i*dim + 1]/2;
 	add_point(&N, igrp, &X, &nmax, point, &groups);
-	for (k = 0; k < nadded[1] - 1; k++){
+	for (double k = 0; k < nadded[1] - 1; k++){
 	  point[0] = p0 + (0.5-drand())*delta[0];
 	  point[1] -= h[1];
 	  add_point(&N, igrp, &X, &nmax, point, &groups);
