@@ -8,9 +8,12 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include "config.h"
+
 #include <limits.h>
 #include <stddef.h>
 #include <util/alloc.h>
+#include <util/list.h>
 #include "selectionfuncs.h"
 #include "topviewfuncs.h"
 #include "smyrna_utils.h"
@@ -107,7 +110,7 @@ static void pick_objects_in_rect(Agraph_t *g, float x1, float y1, float x2,
 static void* pick_object(Agraph_t* g,glCompPoint p)
 {
     double dist = DBL_MAX;
-    float nodeSize=0;
+    double nodeSize = 0;
     void *rv = NULL;
 
     const int defaultNodeShape = getAttrBool(g, g, "defaultnodeshape", 0);
@@ -125,7 +128,7 @@ static void* pick_object(Agraph_t* g,glCompPoint p)
 	}
 
 	// node distance to point
-	const float nd = distBetweenPts(posN , p, nodeSize);
+	const double nd = distBetweenPts(posN , p, nodeSize);
 	if( nd < dist )
 	{
 	    rv=v;dist=nd;
@@ -222,11 +225,11 @@ void deselect_all(Agraph_t* g)
 
 static int close_poly(glCompPoly_t *selPoly, glCompPoint pt) {
     /* int i=0; */
-    const float EPS = GetOGLDistance(3.0f);
-    if (glCompPoly_size(selPoly) < 2)
+    const double EPS = GetOGLDistance(3.0);
+    if (LIST_SIZE(selPoly) < 2)
 	return 0;
-    if (glCompPoly_front(selPoly)->x - pt.x < EPS &&
-        glCompPoly_front(selPoly)->y - pt.y < EPS)
+    if (LIST_FRONT(selPoly)->x - pt.x < EPS &&
+        LIST_FRONT(selPoly)->y - pt.y < EPS)
 	return 1;
     return 0;
 }
@@ -247,12 +250,12 @@ static void select_polygon(Agraph_t *g, glCompPoly_t *selPoly) {
 void add_selpoly(Agraph_t *g, glCompPoly_t *selPoly, glCompPoint pt) {
     if(!close_poly(selPoly,pt))
     {
-	glCompPoly_append(selPoly, (glCompPoint){.x = pt.x, .y = pt.y});
+	LIST_APPEND(selPoly, ((glCompPoint){.x = pt.x, .y = pt.y}));
     }
     else
     {
 	select_polygon (g,selPoly);
-	glCompPoly_free(selPoly);	
+	LIST_FREE(selPoly);	
     }
 }
 
