@@ -14,6 +14,8 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include "config.h"
+
 #include <assert.h>
 #include <common/render.h>
 #include <limits.h>
@@ -23,6 +25,7 @@
 #include <stdio.h>
 #include <util/alloc.h>
 #include <util/exit.h>
+#include <util/gv_math.h>
 #include <util/list.h>
 #include <util/overflow.h>
 #include <util/prisize_t.h>
@@ -41,37 +44,26 @@ static void check_cycles(graph_t * g);
 #define SEQ(a,b,c)		((a) <= (b) && (b) <= (c))
 #define TREE_EDGE(e)	(ED_tree_index(e) >= 0)
 
-DEFINE_LIST(edge_list, edge_t *)
-
 typedef struct {
     graph_t *G;
-    nlist_t Tree_node;
-    edge_list_t Tree_edge;
+    LIST(edge_t *) Tree_edge;
     size_t S_i;			/* search index for enter_edge */
     size_t N_edges, N_nodes;
     int Search_size;
-
-    edge_t *Enter;
-    int Low, Lim, Slack;
 } network_simplex_ctx_t;
 
-#define SEARCHSIZE 30
+enum { SEARCHSIZE = 30 };
 
 static int add_tree_edge(network_simplex_ctx_t *ctx, edge_t * e)
 {
-    node_t *n;
     if (TREE_EDGE(e)) {
 	agerrorf("add_tree_edge: missing tree edge\n");
 	return -1;
     }
-    assert(edge_list_size(&ctx->Tree_edge) <= INT_MAX);
-    ED_tree_index(e) = (int)edge_list_size(&ctx->Tree_edge);
-    edge_list_append(&ctx->Tree_edge, e);
-    if (!ND_mark(agtail(e)))
-	ctx->Tree_node.list[ctx->Tree_node.size++] = agtail(e);
-    if (!ND_mark(aghead(e)))
-	ctx->Tree_node.list[ctx->Tree_node.size++] = aghead(e);
-    n = agtail(e);
+    assert(LIST_SIZE(&ctx->Tree_edge) <= INT_MAX);
+    ED_tree_index(e) = (int)LIST_SIZE(&ctx->Tree_edge);
+    LIST_APPEND(&ctx->Tree_edge, e);
+    node_t *n = agtail(e);
     ND_mark(n) = true;
     ND_tree_out(n).list[ND_tree_out(n).size++] = e;
     ND_tree_out(n).list[ND_tree_out(n).size] = NULL;
@@ -121,14 +113,12 @@ static void invalidate_path(node_t *lca, node_t *to_node) {
 
 static void exchange_tree_edges(network_simplex_ctx_t *ctx, edge_t * e, edge_t * f)
 {
-    node_t *n;
-
     ED_tree_index(f) = ED_tree_index(e);
     assert(ED_tree_index(e) >= 0);
-    edge_list_set(&ctx->Tree_edge, (size_t)ED_tree_index(e), f);
+    LIST_SET(&ctx->Tree_edge, (size_t)ED_tree_index(e), f);
     ED_tree_index(e) = -1;
 
-    n = agtail(e);
+    node_t *n = agtail(e);
     size_t i = --ND_tree_out(n).size;
     size_t j;
     for (j = 0; j <= i; j++)
@@ -152,42 +142,38 @@ static void exchange_tree_edges(network_simplex_ctx_t *ctx, edge_t * e, edge_t *
     ND_tree_in(n).list[ND_tree_in(n).size] = NULL;
 }
 
-DEFINE_LIST(node_queue, node_t *)
-
 static
 void init_rank(network_simplex_ctx_t *ctx)
 {
-    int i;
-    node_t *v;
     edge_t *e;
 
-    node_queue_t Q = {0};
-    node_queue_reserve(&Q, ctx->N_nodes);
+    LIST(node_t *) Q = {0};
+    LIST_RESERVE(&Q, ctx->N_nodes);
     size_t ctr = 0;
 
-    for (v = GD_nlist(ctx->G); v; v = ND_next(v)) {
+    for (node_t *v = GD_nlist(ctx->G); v; v = ND_next(v)) {
 	if (ND_priority(v) == 0)
-	    node_queue_push_back(&Q, v);
+	    LIST_PUSH_BACK(&Q, v);
     }
 
-    while (!node_queue_is_empty(&Q)) {
-	v = node_queue_pop_front(&Q);
+    while (!LIST_IS_EMPTY(&Q)) {
+	node_t *const v = LIST_POP_FRONT(&Q);
 	ND_rank(v) = 0;
 	ctr++;
-	for (i = 0; (e = ND_in(v).list[i]); i++)
+	for (int i = 0; (e = ND_in(v).list[i]); i++)
 	    ND_rank(v) = MAX(ND_rank(v), ND_rank(agtail(e)) + ED_minlen(e));
-	for (i = 0; (e = ND_out(v).list[i]); i++) {
+	for (int i = 0; (e = ND_out(v).list[i]); i++) {
 	    if (--ND_priority(aghead(e)) <= 0)
-		node_queue_push_back(&Q, aghead(e));
+		LIST_PUSH_BACK(&Q, aghead(e));
 	}
     }
     if (ctr != ctx->N_nodes) {
 	agerrorf("trouble in init_rank\n");
-	for (v = GD_nlist(ctx->G); v; v = ND_next(v))
+	for (node_t *v = GD_nlist(ctx->G); v; v = ND_next(v))
 	    if (ND_priority(v))
 		agerr(AGPREV, "\t%s %d\n", agnameof(v), ND_priority(v));
     }
-    node_queue_free(&Q);
+    LIST_FREE(&Q);
 }
 
 static edge_t *leave_edge(network_simplex_ctx_t *ctx)
@@ -196,13 +182,13 @@ static edge_t *leave_edge(network_simplex_ctx_t *ctx)
     int cnt = 0;
 
     size_t j = ctx->S_i;
-    while (ctx->S_i < edge_list_size(&ctx->Tree_edge)) {
-	if (ED_cutvalue(f = edge_list_get(&ctx->Tree_edge, ctx->S_i)) < 0) {
+    while (ctx->S_i < LIST_SIZE(&ctx->Tree_edge)) {
+	if (ED_cutvalue(f = LIST_GET(&ctx->Tree_edge, ctx->S_i)) < 0) {
 	    if (rv) {
 		if (ED_cutvalue(rv) > ED_cutvalue(f))
 		    rv = f;
 	    } else
-		rv = edge_list_get(&ctx->Tree_edge, ctx->S_i);
+		rv = LIST_GET(&ctx->Tree_edge, ctx->S_i);
 	    if (++cnt >= ctx->Search_size)
 		return rv;
 	}
@@ -211,12 +197,12 @@ static edge_t *leave_edge(network_simplex_ctx_t *ctx)
     if (j > 0) {
 	ctx->S_i = 0;
 	while (ctx->S_i < j) {
-	    if (ED_cutvalue(f = edge_list_get(&ctx->Tree_edge, ctx->S_i)) < 0) {
+	    if (ED_cutvalue(f = LIST_GET(&ctx->Tree_edge, ctx->S_i)) < 0) {
 		if (rv) {
 		    if (ED_cutvalue(rv) > ED_cutvalue(f))
 			rv = f;
 		} else
-		    rv = edge_list_get(&ctx->Tree_edge, ctx->S_i);
+		    rv = LIST_GET(&ctx->Tree_edge, ctx->S_i);
 		if (++cnt >= ctx->Search_size)
 		    return rv;
 	    }
@@ -226,52 +212,74 @@ static edge_t *leave_edge(network_simplex_ctx_t *ctx)
     return rv;
 }
 
-static void dfs_enter_outedge(network_simplex_ctx_t *ctx, node_t * v)
-{
-    int i, slack;
+static edge_t *dfs_enter_outedge(node_t *v, int Low, int Lim) {
     edge_t *e;
+    edge_t *Enter = NULL;
+    int Slack = INT_MAX;
 
-    for (i = 0; (e = ND_out(v).list[i]); i++) {
-	if (!TREE_EDGE(e)) {
-	    if (!SEQ(ctx->Low, ND_lim(aghead(e)), ctx->Lim)) {
-		slack = SLACK(e);
-		if (slack < ctx->Slack || ctx->Enter == NULL) {
-		    ctx->Enter = e;
-		    ctx->Slack = slack;
+    LIST(node_t *) todo = {0};
+    LIST_APPEND(&todo, v);
+
+    while (!LIST_IS_EMPTY(&todo)) {
+	v = LIST_POP_BACK(&todo);
+
+	for (int i = 0; (e = ND_out(v).list[i]); i++) {
+	    if (!TREE_EDGE(e)) {
+		if (!SEQ(Low, ND_lim(aghead(e)), Lim)) {
+		    const int slack = SLACK(e);
+		    if (slack < Slack || Enter == NULL) {
+			Enter = e;
+			Slack = slack;
+		    }
 		}
-	    }
-	} else if (ND_lim(aghead(e)) < ND_lim(v))
-	    dfs_enter_outedge(ctx, aghead(e));
+	    } else if (ND_lim(aghead(e)) < ND_lim(v))
+		LIST_APPEND(&todo, aghead(e));
+	}
+	for (int i = 0; (e = ND_tree_in(v).list[i]) && Slack > 0; i++)
+	    if (ND_lim(agtail(e)) < ND_lim(v))
+		LIST_APPEND(&todo, agtail(e));
+
     }
-    for (i = 0; (e = ND_tree_in(v).list[i]) && (ctx->Slack > 0); i++)
-	if (ND_lim(agtail(e)) < ND_lim(v))
-	    dfs_enter_outedge(ctx, agtail(e));
+    LIST_FREE(&todo);
+
+    return Enter;
 }
 
-static void dfs_enter_inedge(network_simplex_ctx_t *ctx, node_t * v)
-{
-    int i, slack;
+static edge_t *dfs_enter_inedge(node_t *v, int Low, int Lim) {
     edge_t *e;
 
-    for (i = 0; (e = ND_in(v).list[i]); i++) {
-	if (!TREE_EDGE(e)) {
-	    if (!SEQ(ctx->Low, ND_lim(agtail(e)), ctx->Lim)) {
-		slack = SLACK(e);
-		if (slack < ctx->Slack || ctx->Enter == NULL) {
-		    ctx->Enter = e;
-		    ctx->Slack = slack;
+    edge_t *Enter = NULL;
+    int Slack = INT_MAX;
+
+    LIST(node_t *) todo = {0};
+    LIST_APPEND(&todo, v);
+
+    while (!LIST_IS_EMPTY(&todo)) {
+	v = LIST_POP_BACK(&todo);
+
+	for (int i = 0; (e = ND_in(v).list[i]); i++) {
+	    if (!TREE_EDGE(e)) {
+		if (!SEQ(Low, ND_lim(agtail(e)), Lim)) {
+		    const int slack = SLACK(e);
+		    if (slack < Slack || Enter == NULL) {
+			Enter = e;
+			Slack = slack;
+		    }
 		}
-	    }
-	} else if (ND_lim(agtail(e)) < ND_lim(v))
-	    dfs_enter_inedge(ctx, agtail(e));
+	    } else if (ND_lim(agtail(e)) < ND_lim(v))
+		LIST_APPEND(&todo, agtail(e));
+	}
+	for (int i = 0; (e = ND_tree_out(v).list[i]) && Slack > 0; i++)
+	    if (ND_lim(aghead(e)) < ND_lim(v))
+		LIST_APPEND(&todo, aghead(e));
+
     }
-    for (i = 0; (e = ND_tree_out(v).list[i]) && ctx->Slack > 0; i++)
-	if (ND_lim(aghead(e)) < ND_lim(v))
-	    dfs_enter_inedge(ctx, aghead(e));
+    LIST_FREE(&todo);
+
+    return Enter;
 }
 
-static edge_t *enter_edge(network_simplex_ctx_t *ctx, edge_t * e)
-{
+static edge_t *enter_edge(edge_t *e) {
     node_t *v;
     bool outsearch;
 
@@ -283,15 +291,9 @@ static edge_t *enter_edge(network_simplex_ctx_t *ctx, edge_t * e)
 	v = aghead(e);
 	outsearch = true;
     }
-    ctx->Enter = NULL;
-    ctx->Slack = INT_MAX;
-    ctx->Low = ND_low(v);
-    ctx->Lim = ND_lim(v);
     if (outsearch)
-	dfs_enter_outedge(ctx, v);
-    else
-	dfs_enter_inedge(ctx, v);
-    return ctx->Enter;
+	return dfs_enter_outedge(v, ND_low(v), ND_lim(v));
+    return dfs_enter_inedge(v, ND_low(v), ND_lim(v));
 }
 
 static void init_cutvalues(network_simplex_ctx_t *ctx)
@@ -325,39 +327,35 @@ typedef struct {
   int rv;    ///< result value
 } tst_t;
 
-/// a stack of states
-DEFINE_LIST(tsts, tst_t)
-
 /* find initial tight subtrees */
 static int tight_subtree_search(network_simplex_ctx_t *ctx, Agnode_t *v, subtree_t *st)
 {
     Agedge_t *e;
-    int     rv;
 
-    rv = 1;
+    int rv = 1;
     ND_subtree_set(v,st);
 
-    tsts_t todo = {0};
-    tsts_push_back(&todo, (tst_t){.v = v, .rv = 1});
+    LIST(tst_t) todo = {0};
+    LIST_PUSH_BACK(&todo, ((tst_t){.v = v, .rv = 1}));
 
-    while (!tsts_is_empty(&todo)) {
+    while (!LIST_IS_EMPTY(&todo)) {
         bool updated = false;
-        tst_t *top = tsts_back(&todo);
+        tst_t *top = LIST_BACK(&todo);
 
         for (; (e = ND_in(top->v).list[top->in_i]); top->in_i++) {
             if (TREE_EDGE(e)) continue;
             if (ND_subtree(agtail(e)) == 0 && SLACK(e) == 0) {
                 if (add_tree_edge(ctx, e) != 0) {
-                    (void)tsts_pop_back(&todo);
-                    if (tsts_is_empty(&todo)) {
+                    LIST_DROP_BACK(&todo);
+                    if (LIST_IS_EMPTY(&todo)) {
                         rv = -1;
                     } else {
-                        --tsts_back(&todo)->rv;
+                        --LIST_BACK(&todo)->rv;
                     }
                 } else {
                     ND_subtree_set(agtail(e), st);
                     const tst_t next = {.v = agtail(e), .rv = 1};
-                    tsts_push_back(&todo, next);
+                    LIST_PUSH_BACK(&todo, next);
                 }
                 updated = true;
                 break;
@@ -371,16 +369,16 @@ static int tight_subtree_search(network_simplex_ctx_t *ctx, Agnode_t *v, subtree
             if (TREE_EDGE(e)) continue;
             if (ND_subtree(aghead(e)) == 0 && SLACK(e) == 0) {
                 if (add_tree_edge(ctx, e) != 0) {
-                    (void)tsts_pop_back(&todo);
-                    if (tsts_is_empty(&todo)) {
+                    LIST_DROP_BACK(&todo);
+                    if (LIST_IS_EMPTY(&todo)) {
                         rv = -1;
                     } else {
-                        --tsts_back(&todo)->rv;
+                        --LIST_BACK(&todo)->rv;
                     }
                 } else {
                     ND_subtree_set(aghead(e), st);
                     const tst_t next = {.v = aghead(e), .rv = 1};
-                    tsts_push_back(&todo, next);
+                    LIST_PUSH_BACK(&todo, next);
                 }
                 updated = true;
                 break;
@@ -390,23 +388,22 @@ static int tight_subtree_search(network_simplex_ctx_t *ctx, Agnode_t *v, subtree
           continue;
         }
 
-        const tst_t last = tsts_pop_back(&todo);
-        if (tsts_is_empty(&todo)) {
+        const tst_t last = LIST_POP_BACK(&todo);
+        if (LIST_IS_EMPTY(&todo)) {
             rv = last.rv;
         } else {
-            tsts_back(&todo)->rv += last.rv;
+            LIST_BACK(&todo)->rv += last.rv;
         }
     }
 
-    tsts_free(&todo);
+    LIST_FREE(&todo);
 
     return rv;
 }
 
 static subtree_t *find_tight_subtree(network_simplex_ctx_t *ctx, Agnode_t *v)
 {
-    subtree_t       *rv;
-    rv = gv_alloc(sizeof(subtree_t));
+    subtree_t *rv = gv_alloc(sizeof(subtree_t));
     rv->rep = v;
     rv->size = tight_subtree_search(ctx,v,rv);
     if (rv->size < 0) {
@@ -454,11 +451,10 @@ static subtree_t *STsetUnion(subtree_t *s0, subtree_t *s1)
 /* find tightest edge to another tree incident on the given tree */
 static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *best)
 {
-    int i;
     Agedge_t *e;
     subtree_t *ts = STsetFind(v);
     if (best && SLACK(best) == 0) return best;
-    for (i = 0; (e = ND_out(v).list[i]); i++) {
+    for (int i = 0; (e = ND_out(v).list[i]); i++) {
       if (TREE_EDGE(e)) {
           if (aghead(e) == from) continue;  // do not search back in tree
           best = inter_tree_edge_search(aghead(e),v,best); // search forward in tree
@@ -471,7 +467,7 @@ static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *b
       }
     }
     /* the following code must mirror the above, but for in-edges */
-    for (i = 0; (e = ND_in(v).list[i]); i++) {
+    for (int i = 0; (e = ND_in(v).list[i]); i++) {
       if (TREE_EDGE(e)) {
           if (agtail(e) == from) continue;
           best = inter_tree_edge_search(agtail(e),v,best);
@@ -487,9 +483,7 @@ static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *b
 
 static Agedge_t *inter_tree_edge(subtree_t *tree)
 {
-    Agedge_t *rv;
-    rv = inter_tree_edge_search(tree->rep, NULL, NULL);
-    return rv;
+    return inter_tree_edge_search(tree->rep, NULL, NULL);
 }
 
 static size_t STheapsize(const STheap_t *heap) { return heap->size; }
@@ -503,10 +497,7 @@ static void STheapify(STheap_t *heap, size_t i) {
         if (left < heap->size && elt[left]->size < elt[smallest]->size) smallest = left;
         if (right < heap->size && elt[right]->size < elt[smallest]->size) smallest = right;
         if (smallest != i) {
-            subtree_t *temp;
-            temp = elt[i];
-            elt[i] = elt[smallest];
-            elt[smallest] = temp;
+            SWAP(&elt[i], &elt[smallest]);
             elt[i]->heap_index = i;
             elt[smallest]->heap_index = smallest;
             i = smallest;
@@ -528,8 +519,7 @@ static STheap_t *STbuildheap(subtree_t **elt, size_t size) {
 static
 subtree_t *STextractmin(STheap_t *heap)
 {
-    subtree_t *rv;
-    rv = heap->elt[0];
+    subtree_t *rv = heap->elt[0];
     rv->heap_index = SIZE_MAX;
       // mark this as not participating in the heap anymore
     heap->elt[0] = heap->elt[heap->size - 1];
@@ -543,17 +533,15 @@ subtree_t *STextractmin(STheap_t *heap)
 static
 void tree_adjust(Agnode_t *v, Agnode_t *from, int delta)
 {
-    int i;
     Agedge_t *e;
-    Agnode_t *w;
-    ND_rank(v) = ND_rank(v) + delta;
-    for (i = 0; (e = ND_tree_in(v).list[i]); i++) {
-      w = agtail(e);
+    ND_rank(v) += delta;
+    for (int i = 0; (e = ND_tree_in(v).list[i]); i++) {
+      Agnode_t *const w = agtail(e);
       if (w != from)
         tree_adjust(w, v, delta);
     }
-    for (i = 0; (e = ND_tree_out(v).list[i]); i++) {
-      w = aghead(e);
+    for (int i = 0; (e = ND_tree_out(v).list[i]); i++) {
+      Agnode_t *const w = aghead(e);
       if (w != from)
         tree_adjust(w, v, delta);
     }
@@ -562,35 +550,30 @@ void tree_adjust(Agnode_t *v, Agnode_t *from, int delta)
 static
 subtree_t *merge_trees(network_simplex_ctx_t *ctx, Agedge_t *e)   /* entering tree edge */
 {
-  int       delta;
-  subtree_t *t0, *t1, *rv;
-
   assert(!TREE_EDGE(e));
 
-  t0 = STsetFind(agtail(e));
-  t1 = STsetFind(aghead(e));
+  subtree_t *const t0 = STsetFind(agtail(e));
+  subtree_t *const t1 = STsetFind(aghead(e));
 
   if (!on_heap(t0)) { // move t0
-    delta = SLACK(e);
+    const int delta = SLACK(e);
     if (delta != 0)
       tree_adjust(t0->rep,NULL,delta);
   }
   else {  // move t1
-    delta = -SLACK(e);
+    const int delta = -SLACK(e);
     if (delta != 0)
       tree_adjust(t1->rep,NULL,delta);
   }
   if (add_tree_edge(ctx, e) != 0) {
     return NULL;
   }
-  rv = STsetUnion(t0,t1);
-  
-  return rv;
+  return STsetUnion(t0, t1);
 }
 
 /* Construct initial tight tree. Graph must be connected, feasible.
  * Adjust ND_rank(v) as needed.  add_tree_edge() on tight tree edges.
- * trees are basically lists of nodes stored in `node_queue_t`s.
+ * trees are basically lists of nodes stored in `LIST(node_t *)`s.
  * Return 1 if input graph is not connected; 0 on success.
  */
 static
@@ -640,23 +623,16 @@ end:
   for (size_t i = 0; i < subtree_count; i++) free(tree[i]);
   free(tree);
   if (error) return error;
-  assert(edge_list_size(&ctx->Tree_edge) == ctx->N_nodes - 1);
+  assert(LIST_SIZE(&ctx->Tree_edge) == ctx->N_nodes - 1);
   init_cutvalues(ctx);
   return 0;
 }
 
 /* walk up from v to LCA(v,w), setting new cutvalues. */
-static Agnode_t *treeupdate(Agnode_t * v, Agnode_t * w, int cutvalue, int dir)
-{
-    edge_t *e;
-    int d;
-
+static Agnode_t *treeupdate(Agnode_t *v, Agnode_t *w, int cutvalue, bool dir) {
     while (!SEQ(ND_low(v), ND_lim(w), ND_lim(v))) {
-	e = ND_par(v);
-	if (v == agtail(e))
-	    d = dir;
-	else
-	    d = !dir;
+	edge_t *const e = ND_par(v);
+	const bool d = v == agtail(e) ? dir : !dir;
 	if (d)
 	    ED_cutvalue(e) += cutvalue;
 	else
@@ -671,14 +647,13 @@ static Agnode_t *treeupdate(Agnode_t * v, Agnode_t * w, int cutvalue, int dir)
 
 static void rerank(Agnode_t * v, int delta)
 {
-    int i;
     edge_t *e;
 
     ND_rank(v) -= delta;
-    for (i = 0; (e = ND_tree_out(v).list[i]); i++)
+    for (int i = 0; (e = ND_tree_out(v).list[i]); i++)
 	if (e != ND_par(v))
 	    rerank(aghead(e), delta);
-    for (i = 0; (e = ND_tree_in(v).list[i]); i++)
+    for (int i = 0; (e = ND_tree_in(v).list[i]); i++)
 	if (e != ND_par(v))
 	    rerank(agtail(e), delta);
 }
@@ -689,10 +664,7 @@ static void rerank(Agnode_t * v, int delta)
 static int
 update(network_simplex_ctx_t *ctx, edge_t * e, edge_t * f)
 {
-    int cutvalue, delta;
-    Agnode_t *lca;
-
-    delta = SLACK(f);
+    const int delta = SLACK(f);
     /* "for (v = in nodes in tail side of e) do ND_rank(v) -= delta;" */
     if (delta > 0) {
 	size_t s = ND_tree_in(agtail(e)).size + ND_tree_out(agtail(e)).size;
@@ -711,9 +683,9 @@ update(network_simplex_ctx_t *ctx, edge_t * e, edge_t * f)
 	}
     }
 
-    cutvalue = ED_cutvalue(e);
-    lca = treeupdate(agtail(f), aghead(f), cutvalue, 1);
-    if (treeupdate(aghead(f), agtail(f), cutvalue, 0) != lca) {
+    const int cutvalue = ED_cutvalue(e);
+    Agnode_t *const lca = treeupdate(agtail(f), aghead(f), cutvalue, true);
+    if (treeupdate(aghead(f), agtail(f), cutvalue, false) != lca) {
 	agerrorf("update: mismatched lca in treeupdates\n");
 	return 2;
     }
@@ -731,35 +703,28 @@ update(network_simplex_ctx_t *ctx, edge_t * e, edge_t * f)
 }
 
 static int scan_and_normalize(network_simplex_ctx_t *ctx) {
-    node_t *n;
-
     int Minrank = INT_MAX;
     int Maxrank = INT_MIN;
-    for (n = GD_nlist(ctx->G); n; n = ND_next(n)) {
+    for (node_t *n = GD_nlist(ctx->G); n; n = ND_next(n)) {
 	if (ND_node_type(n) == NORMAL) {
 	    Minrank = MIN(Minrank, ND_rank(n));
 	    Maxrank = MAX(Maxrank, ND_rank(n));
 	}
     }
-    for (n = GD_nlist(ctx->G); n; n = ND_next(n))
+    for (node_t *n = GD_nlist(ctx->G); n; n = ND_next(n))
 	ND_rank(n) -= Minrank;
     Maxrank -= Minrank;
     return Maxrank;
 }
 
 static void reset_lists(network_simplex_ctx_t *ctx) {
-
-  free(ctx->Tree_node.list);
-  ctx->Tree_node = (nlist_t){0};
-
-  edge_list_free(&ctx->Tree_edge);
+  LIST_FREE(&ctx->Tree_edge);
 }
 
 static void
 freeTreeList (network_simplex_ctx_t *ctx, graph_t* g)
 {
-    node_t *n;
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	free_list(ND_tree_in(n));
 	free_list(ND_tree_out(n));
 	ND_mark(n) = false;
@@ -769,16 +734,13 @@ freeTreeList (network_simplex_ctx_t *ctx, graph_t* g)
 
 static void LR_balance(network_simplex_ctx_t *ctx)
 {
-    int delta;
-    edge_t *e, *f;
-
-    for (size_t i = 0; i < edge_list_size(&ctx->Tree_edge); i++) {
-	e = edge_list_get(&ctx->Tree_edge, i);
+    for (size_t i = 0; i < LIST_SIZE(&ctx->Tree_edge); i++) {
+	edge_t *const e = LIST_GET(&ctx->Tree_edge, i);
 	if (ED_cutvalue(e) == 0) {
-	    f = enter_edge(ctx,e);
+	    edge_t *const f = enter_edge(e);
 	    if (f == NULL)
 		continue;
-	    delta = SLACK(f);
+	    const int delta = SLACK(f);
 	    if (delta <= 1)
 		continue;
 	    if (ND_lim(agtail(e)) < ND_lim(aghead(e)))
@@ -791,18 +753,8 @@ static void LR_balance(network_simplex_ctx_t *ctx)
 }
 
 static int decreasingrankcmpf(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  node_t **n0 = (node_t**)x;
-  node_t **n1 = (node_t**)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+  node_t *const *n0 = x;
+  node_t *const *n1 = y;
   if (ND_rank(*n1) < ND_rank(*n0)) {
     return -1;
   }
@@ -813,33 +765,12 @@ static int decreasingrankcmpf(const void *x, const void *y) {
 }
 
 static int increasingrankcmpf(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  node_t **n0 = (node_t **)x;
-  node_t **n1 = (node_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-  if (ND_rank(*n0) < ND_rank(*n1)) {
-    return -1;
-  }
-  if (ND_rank(*n0) > ND_rank(*n1)) {
-    return 1;
-  }
-  return 0;
+  return -decreasingrankcmpf(x, y);
 }
 
 static void TB_balance(network_simplex_ctx_t *ctx)
 {
-    node_t *n;
     edge_t *e;
-    int low, high, choice;
-    int inweight, outweight;
     int adj = 0;
     char *s;
 
@@ -851,7 +782,7 @@ static void TB_balance(network_simplex_ctx_t *ctx)
     if ( (s = agget(ctx->G,"TBbalance")) ) {
          if (streq(s,"min")) adj = 1;
          else if (streq(s,"max")) adj = 2;
-         if (adj) for (n = GD_nlist(ctx->G); n; n = ND_next(n))
+         if (adj) for (node_t *n = GD_nlist(ctx->G); n; n = ND_next(n))
               if (ND_node_type(n) == NORMAL) {
                 if (ND_in(n).size == 0 && adj == 1) {
                    ND_rank(n) = 0;
@@ -861,25 +792,25 @@ static void TB_balance(network_simplex_ctx_t *ctx)
                 }
               }
     }
-    size_t ii;
-    for (ii = 0, n = GD_nlist(ctx->G); n; ii++, n = ND_next(n)) {
-      ctx->Tree_node.list[ii] = n;
+    LIST(node_t *) Tree_node = {0};
+    LIST_RESERVE(&Tree_node, ctx->N_nodes);
+    for (node_t *n = GD_nlist(ctx->G); n; n = ND_next(n)) {
+      LIST_APPEND(&Tree_node, n);
     }
-    ctx->Tree_node.size = ii;
-    qsort(ctx->Tree_node.list, ctx->Tree_node.size, sizeof(ctx->Tree_node.list[0]),
-          adj > 1 ? decreasingrankcmpf: increasingrankcmpf);
-    for (size_t i = 0; i < ctx->Tree_node.size; i++) {
-        n = ctx->Tree_node.list[i];
+    LIST_SORT(&Tree_node, adj > 1 ? decreasingrankcmpf: increasingrankcmpf);
+    for (size_t i = 0; i < LIST_SIZE(&Tree_node); i++) {
+        node_t *const n = LIST_GET(&Tree_node, i);
         if (ND_node_type(n) == NORMAL)
           nrank[ND_rank(n)]++;
     }
-    for (ii = 0; ii < ctx->Tree_node.size; ii++) {
-      n = ctx->Tree_node.list[ii];
+    for (size_t ii = 0; ii < LIST_SIZE(&Tree_node); ii++) {
+      node_t *const n = LIST_GET(&Tree_node, ii);
       if (ND_node_type(n) != NORMAL)
         continue;
-      inweight = outweight = 0;
-      low = 0;
-      high = Maxrank;
+      int inweight = 0;
+      int outweight = 0;
+      int low = 0;
+      int high = Maxrank;
       for (size_t i = 0; (e = ND_in(n).list[i]); i++) {
         inweight += ED_weight(e);
         low = MAX(low, ND_rank(agtail(e)) + ED_minlen(e));
@@ -892,11 +823,11 @@ static void TB_balance(network_simplex_ctx_t *ctx)
         low = 0;		/* vnodes can have ranks < 0 */
       if (adj) {
         if (inweight == outweight)
-            ND_rank(n) = (adj == 1? low : high);
+            ND_rank(n) = adj == 1 ? low : high;
       }
       else {
                 if (inweight == outweight) {
-                    choice = low;
+                    int choice = low;
                     for (int i = low + 1; i <= high; i++)
                         if (nrank[i] < nrank[choice])
                             choice = i;
@@ -909,27 +840,25 @@ static void TB_balance(network_simplex_ctx_t *ctx)
       free_list(ND_tree_out(n));
       ND_mark(n) = false;
     }
+    LIST_FREE(&Tree_node);
     free(nrank);
 }
 
 static bool init_graph(network_simplex_ctx_t *ctx, graph_t *g) {
-    node_t *n;
     edge_t *e;
 
-    ctx->G = g;
-    ctx->N_nodes = ctx->N_edges = ctx->S_i = 0;
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    *ctx = (network_simplex_ctx_t){.G = g};
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	ND_mark(n) = false;
 	ctx->N_nodes++;
-	for (size_t i = 0; (e = ND_out(n).list[i]); i++)
+	for (size_t i = 0; ND_out(n).list[i]; i++)
 	    ctx->N_edges++;
     }
 
-    ctx->Tree_node.list = gv_calloc(ctx->N_nodes, sizeof(node_t *));
-    edge_list_reserve(&ctx->Tree_edge, ctx->N_nodes);
+    LIST_RESERVE(&ctx->Tree_edge, ctx->N_nodes);
 
     bool feasible = true;
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	ND_priority(n) = 0;
 	size_t i;
 	for (i = 0; (e = ND_in(n).list[i]); i++) {
@@ -941,7 +870,7 @@ static bool init_graph(network_simplex_ctx_t *ctx, graph_t *g) {
 	}
 	ND_tree_in(n).list = gv_calloc(i + 1, sizeof(edge_t *));
 	ND_tree_in(n).size = 0;
-	for (i = 0; (e = ND_out(n).list[i]); i++);
+	for (i = 0; ND_out(n).list[i]; i++);
 	ND_tree_out(n).list = gv_calloc(i + 1, sizeof(edge_t *));
 	ND_tree_out(n).size = 0;
     }
@@ -951,17 +880,12 @@ static bool init_graph(network_simplex_ctx_t *ctx, graph_t *g) {
 /* graphSize:
  * Compute no. of nodes and edges in the graph
  */
-static void
-graphSize (graph_t * g, int* nn, int* ne)
-{
-    int i, nnodes, nedges;
-    node_t *n;
-    edge_t *e;
-   
-    nnodes = nedges = 0;
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+static void graphSize(graph_t *g, size_t *nn, size_t *ne) {
+    size_t nnodes = 0;
+    size_t nedges = 0;
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	nnodes++;
-	for (i = 0; (e = ND_out(n).list[i]); i++) {
+	for (size_t i = 0; ND_out(n).list[i]; i++) {
 	    nedges++;
 	}
     }
@@ -984,18 +908,18 @@ graphSize (graph_t * g, int* nn, int* ne)
 int rank2(graph_t * g, int balance, int maxiter, int search_size)
 {
     int iter = 0;
-    char *ns = "network simplex: ";
-    edge_t *e, *f;
+    char ns[] = "network simplex: ";
+    edge_t *e;
     network_simplex_ctx_t ctx = {0};
 
 #ifdef DEBUG
     check_cycles(g);
 #endif
     if (Verbose) {
-	int nn, ne;
+	size_t nn, ne;
 	graphSize (g, &nn, &ne);
-	fprintf(stderr, "%s %d nodes %d edges maxiter=%d balance=%d\n", ns,
-	    nn, ne, maxiter, balance);
+	fprintf(stderr, "%s %" PRISIZE_T " nodes %" PRISIZE_T
+	        " edges maxiter=%d balance=%d\n", ns, nn, ne, maxiter, balance);
 	start_timer();
     }
     bool feasible = init_graph(&ctx, g);
@@ -1008,7 +932,7 @@ int rank2(graph_t * g, int balance, int maxiter, int search_size)
 	ctx.Search_size = SEARCHSIZE;
 
     {
-	int err = feasible_tree(&ctx);
+	const int err = feasible_tree(&ctx);
 	if (err != 0) {
 	    freeTreeList(&ctx, g);
 	    return err;
@@ -1020,9 +944,8 @@ int rank2(graph_t * g, int balance, int maxiter, int search_size)
     }
 
     while ((e = leave_edge(&ctx))) {
-	int err;
-	f = enter_edge(&ctx, e);
-	err = update(&ctx, e, f);
+	edge_t *const f = enter_edge(e);
+	const int err = update(&ctx, e, f);
 	if (err != 0) {
 	    freeTreeList(&ctx, g);
 	    return err;
@@ -1078,7 +1001,7 @@ static void x_cutval(edge_t * f)
 {
     node_t *v;
     edge_t *e;
-    int i, sum, dir;
+    int dir;
 
     /* set v to the node on the side of the edge already searched */
     if (ND_par(agtail(f)) == f) {
@@ -1089,13 +1012,13 @@ static void x_cutval(edge_t * f)
 	dir = -1;
     }
 
-    sum = 0;
-    for (i = 0; (e = ND_out(v).list[i]); i++)
+    int sum = 0;
+    for (int i = 0; (e = ND_out(v).list[i]); i++)
 	if (sadd_overflow(sum, x_val(e, v, dir), &sum)) {
 	    agerrorf("overflow when computing edge weight sum\n");
 	    graphviz_exit(EXIT_FAILURE);
 	}
-    for (i = 0; (e = ND_in(v).list[i]); i++)
+    for (int i = 0; (e = ND_in(v).list[i]); i++)
 	if (sadd_overflow(sum, x_val(e, v, dir), &sum)) {
 	    agerrorf("overflow when computing edge weight sum\n");
 	    graphviz_exit(EXIT_FAILURE);
@@ -1143,13 +1066,12 @@ static int x_val(edge_t * e, node_t * v, int dir)
 
 static void dfs_cutval(node_t * v, edge_t * par)
 {
-    int i;
     edge_t *e;
 
-    for (i = 0; (e = ND_tree_out(v).list[i]); i++)
+    for (int i = 0; (e = ND_tree_out(v).list[i]); i++)
 	if (e != par)
 	    dfs_cutval(aghead(e), e);
-    for (i = 0; (e = ND_tree_in(v).list[i]); i++)
+    for (int i = 0; (e = ND_tree_in(v).list[i]); i++)
 	if (e != par)
 	    dfs_cutval(agtail(e), e);
     if (par)
@@ -1165,8 +1087,6 @@ typedef struct {
   int tree_in_i;
 } dfs_state_t;
 
-DEFINE_LIST(dfs_stack, dfs_state_t)
-
 /*
 * Initializes DFS range attributes (par, low, lim) over tree nodes such that:
 * ND_par(n) - parent tree edge
@@ -1176,16 +1096,16 @@ DEFINE_LIST(dfs_stack, dfs_state_t)
 static int dfs_range_init(node_t *v) {
     int lim = 0;
 
-    dfs_stack_t todo = {0};
+    LIST(dfs_state_t) todo = {0};
 
     ND_par(v) = NULL;
     ND_low(v) = 1;
     const dfs_state_t root = {.v = v, .par = NULL, .lim = 1};
-    dfs_stack_push_back(&todo, root);
+    LIST_PUSH_BACK(&todo, root);
 
-    while (!dfs_stack_is_empty(&todo)) {
+    while (!LIST_IS_EMPTY(&todo)) {
         bool pushed_new = false;
-        dfs_state_t *const s = dfs_stack_back(&todo);
+        dfs_state_t *const s = LIST_BACK(&todo);
 
         while (ND_tree_out(s->v).list[s->tree_out_i]) {
             edge_t *const e = ND_tree_out(s->v).list[s->tree_out_i];
@@ -1195,7 +1115,7 @@ static int dfs_range_init(node_t *v) {
                 ND_par(n) = e;
                 ND_low(n) = s->lim;
                 const dfs_state_t next = {.v = n, .par = e, .lim = s->lim};
-                dfs_stack_push_back(&todo, next);
+                LIST_PUSH_BACK(&todo, next);
                 pushed_new = true;
                 break;
             }
@@ -1212,7 +1132,7 @@ static int dfs_range_init(node_t *v) {
                 ND_par(n) = e;
                 ND_low(n) = s->lim;
                 const dfs_state_t next = {.v = n, .par = e, .lim = s->lim};
-                dfs_stack_push_back(&todo, next);
+                LIST_PUSH_BACK(&todo, next);
                 pushed_new = true;
                 break;
             }
@@ -1224,14 +1144,14 @@ static int dfs_range_init(node_t *v) {
         ND_lim(s->v) = s->lim;
 
         lim = s->lim;
-        (void)dfs_stack_pop_back(&todo);
+        LIST_DROP_BACK(&todo);
 
-        if (!dfs_stack_is_empty(&todo)) {
-            dfs_stack_back(&todo)->lim = lim + 1;
+        if (!LIST_IS_EMPTY(&todo)) {
+            LIST_BACK(&todo)->lim = lim + 1;
         }
     }
 
-    dfs_stack_free(&todo);
+    LIST_FREE(&todo);
 
     return lim + 1;
 }
@@ -1247,16 +1167,16 @@ static int dfs_range(node_t * v, edge_t * par, int low)
 	return ND_lim(v) + 1;
     }
 
-    dfs_stack_t todo = {0};
+    LIST(dfs_state_t) todo = {0};
 
     ND_par(v) = par;
     ND_low(v) = low;
     const dfs_state_t root = {.v = v, .par = par, .lim = low};
-    dfs_stack_push_back(&todo, root);
+    LIST_PUSH_BACK(&todo, root);
 
-    while (!dfs_stack_is_empty(&todo)) {
+    while (!LIST_IS_EMPTY(&todo)) {
 	bool processed_child = false;
-	dfs_state_t *const s = dfs_stack_back(&todo);
+	dfs_state_t *const s = LIST_BACK(&todo);
 
 	while (ND_tree_out(s->v).list[s->tree_out_i]) {
 	    edge_t *const e = ND_tree_out(s->v).list[s->tree_out_i];
@@ -1269,7 +1189,7 @@ static int dfs_range(node_t * v, edge_t * par, int low)
 		    ND_par(n) = e;
 		    ND_low(n) = s->lim;
 		    const dfs_state_t next = {.v = n, .par = e, .lim = s->lim};
-		    dfs_stack_push_back(&todo, next);
+		    LIST_PUSH_BACK(&todo, next);
 		}
 		processed_child = true;
 		break;
@@ -1290,7 +1210,7 @@ static int dfs_range(node_t * v, edge_t * par, int low)
 		    ND_par(n) = e;
 		    ND_low(n) = s->lim;
 		    const dfs_state_t next = {.v = n, .par = e, .lim = s->lim};
-		    dfs_stack_push_back(&todo, next);
+		    LIST_PUSH_BACK(&todo, next);
 		}
 		processed_child = true;
 		break;
@@ -1303,14 +1223,14 @@ static int dfs_range(node_t * v, edge_t * par, int low)
 	ND_lim(s->v) = s->lim;
 
 	lim = s->lim;
-	(void)dfs_stack_pop_back(&todo);
+	LIST_DROP_BACK(&todo);
 
-	if (!dfs_stack_is_empty(&todo)) {
-	    dfs_stack_back(&todo)->lim = lim + 1;
+	if (!LIST_IS_EMPTY(&todo)) {
+	    LIST_BACK(&todo)->lim = lim + 1;
 	}
     }
 
-    dfs_stack_free(&todo);
+    LIST_FREE(&todo);
 
     return lim + 1;
 }
@@ -1318,33 +1238,22 @@ static int dfs_range(node_t * v, edge_t * par, int low)
 #ifdef DEBUG
 void tchk(network_simplex_ctx_t *ctx)
 {
-    int i;
-    node_t *n;
     edge_t *e;
 
     size_t n_cnt = 0;
     size_t e_cnt = 0;
-    for (n = agfstnode(ctx->G); n; n = agnxtnode(ctx->G, n)) {
+    for (node_t *n = agfstnode(ctx->G); n; n = agnxtnode(ctx->G, n)) {
 	n_cnt++;
-	for (i = 0; (e = ND_tree_out(n).list[i]); i++) {
+	for (int i = 0; (e = ND_tree_out(n).list[i]); i++) {
 	    e_cnt++;
 	    if (SLACK(e) > 0)
 		fprintf(stderr, "not a tight tree %p", e);
 	}
     }
-    if (n_cnt != ctx->Tree_node.size || e_cnt != edge_list_size(&ctx->Tree_edge))
+    if (e_cnt != LIST_SIZE(&ctx->Tree_edge))
 	fprintf(stderr, "something missing\n");
 }
 
-void check_fast_node(node_t * n)
-{
-    node_t *nptr;
-    nptr = GD_nlist(agraphof(n));
-    while (nptr && nptr != n)
-	nptr = ND_next(nptr);
-    assert(nptr != NULL);
-}
-
 static void dump_node(FILE *sink, node_t *n) {
     if (ND_node_type(n)) {
       fprintf(sink, "%p", n);
@@ -1355,22 +1264,20 @@ static void dump_node(FILE *sink, node_t *n) {
 
 static void dump_graph (graph_t* g)
 {
-    int i;
     edge_t *e;
-    node_t *n,*w;
-    FILE* fp = fopen ("ns.gv", "w");
+    FILE *const fp = fopen ("ns.gv", "w");
     fprintf (fp, "digraph \"%s\" {\n", agnameof(g));
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	fputs("  \"", fp);
 	dump_node(fp, n);
 	fputs("\"\n", fp);
     }
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
-	for (i = 0; (e = ND_out(n).list[i]); i++) {
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
+	for (int i = 0; (e = ND_out(n).list[i]); i++) {
 	    fputs("  \"", fp);
 	    dump_node(fp, n);
 	    fputs("\"", fp);
-	    w = aghead(e);
+	    node_t *const w = aghead(e);
 	    fputs(" -> \"", fp);
 	    dump_node(fp, w);
 	    fputs("\"\n", fp);
@@ -1384,51 +1291,44 @@ static void dump_graph (graph_t* g)
 static node_t *checkdfs(graph_t* g, node_t * n)
 {
     edge_t *e;
-    node_t *w,*x;
-    int i;
 
     if (ND_mark(n))
-	return 0;
+	return NULL;
     ND_mark(n) = true;
     ND_onstack(n) = true;
-    for (i = 0; (e = ND_out(n).list[i]); i++) {
-	w = aghead(e);
+    for (int i = 0; (e = ND_out(n).list[i]); i++) {
+	node_t *const w = aghead(e);
 	if (ND_onstack(w)) {
 	    dump_graph (g);
-	    fprintf(stderr, "cycle: last edge %lx %s(%lx) %s(%lx)\n",
-		(uint64_t)e,
-	       	agnameof(n), (uint64_t)n,
-		agnameof(w), (uint64_t)w);
+	    fprintf(stderr, "cycle: last edge %p %s(%p) %s(%p)\n", e, agnameof(n), n,
+	            agnameof(w), w);
 	    return w;
 	}
 	else {
 	    if (!ND_mark(w)) {
-		x = checkdfs(g, w);
+		node_t *const x = checkdfs(g, w);
 		if (x) {
-		    fprintf(stderr,"unwind %lx %s(%lx)\n",
-			(uint64_t)e,
-			agnameof(n), (uint64_t)n);
+		    fprintf(stderr,"unwind %p %s(%p)\n", e, agnameof(n), n);
 		    if (x != n) return x;
 		    fprintf(stderr,"unwound to root\n");
 		    fflush(stderr);
 		    abort();
-		    return 0;
+		    return NULL;
 		}
 	    }
 	}
     }
     ND_onstack(n) = false;
-    return 0;
+    return NULL;
 }
 
 void check_cycles(graph_t * g)
 {
-    node_t *n;
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n)) {
 	ND_mark(n) = false;
 	ND_onstack(n) = false;
     }
-    for (n = GD_nlist(g); n; n = ND_next(n))
+    for (node_t *n = GD_nlist(g); n; n = ND_next(n))
 	checkdfs(g, n);
 }
 #endif				/* DEBUG */
