@@ -16,7 +16,10 @@
  * an edge, starting from a list of control points.
  */
 
+#include "config.h"
+
 #include <math.h>
+#include <common/geomprocs.h>
 #include <common/render.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -24,6 +27,7 @@
 #include <util/agxbuf.h>
 #include <util/alloc.h>
 #include <util/gv_math.h>
+#include <util/streq.h>
 #include <util/unreachable.h>
 
 #ifdef DEBUG
@@ -51,8 +55,7 @@ static void showPoints(pointf ps[], int pn)
 }
 #endif
 
-/* arrow_clip:
- * Clip arrow to node boundary.
+/* Clip arrow to node boundary.
  * The real work is done elsewhere. Here we get the real edge,
  * check that the edge has arrowheads, and that an endpoint
  * isn't a merge point where several parts of an edge meet.
@@ -96,8 +99,7 @@ arrow_clip(edge_t * fe, node_t * hn,
     }
 }
 
-/* bezier_clip
- * Clip bezier to shape using binary search.
+/* Clip Bézier to shape using binary search.
  * The details of the shape are passed in the inside_context;
  * The function providing the inside test is passed as a parameter.
  * left_inside specifies that sp[0] is inside the node,
@@ -150,8 +152,7 @@ void bezier_clip(inside_t * inside_context,
 	    sp[i] = seg[i];
 }
 
-/* shape_clip0:
- * Clip Bezier to node shape using binary search.
+/* Clip Bézier to node shape using binary search.
  * left_inside specifies that curve[0] is inside the node, else
  * curve[3] is taken as inside.
  * Assumes ND_shape(n) and ND_shape(n)->fns->insidefn are non-NULL.
@@ -180,12 +181,11 @@ shape_clip0(inside_t * inside_context, node_t * n, pointf curve[4],
     ND_rw(n) = save_real_size;
 }
 
-/* shape_clip:
- * Clip Bezier to node shape
+/* Clip Bézier to node shape
  * Uses curve[0] to determine which which side is inside the node.
  * NOTE: This test is bad. It is possible for previous call to
- * shape_clip to produce a Bezier with curve[0] moved to the boundary
- * for which insidefn(curve[0]) is true. Thus, if the new Bezier is
+ * shape_clip to produce a Bézier with curve[0] moved to the boundary
+ * for which insidefn(curve[0]) is true. Thus, if the new Bézier is
  * fed back to shape_clip, it will again assume left_inside is true.
  * To be safe, shape_clip0 should guarantee that the computed boundary
  * point fails insidefn.
@@ -210,9 +210,7 @@ void shape_clip(node_t * n, pointf curve[4])
     shape_clip0(&inside_context, n, curve, left_inside);
 }
 
-/* new_spline:
- * Create and attach a new bezier of size sz to the edge d
- */
+/// create and attach a new Bézier of size sz to the edge d
 bezier *new_spline(edge_t *e, size_t sz) {
     bezier *rv;
     while (ED_to_orig(e) != NULL && ED_edge_type(e) != NORMAL)
@@ -229,8 +227,7 @@ bezier *new_spline(edge_t *e, size_t sz) {
     return rv;
 }
 
-/* clip_and_install:
- * Given a raw spline (pn control points in ps), representing
+/* Given a raw spline (pn control points in ps), representing
  * a path from edge agtail(fe) ending in node hn, clip the ends to
  * the node boundaries and attach the resulting spline to the
  * edge.
@@ -344,8 +341,7 @@ void add_box(path * P, boxf b)
 	P->boxes[P->nbox++] = b;
 }
 
-/* beginpath:
- * Set up boxes near the tail node.
+/* Set up boxes near the tail node.
  * For regular nodes, the result should be a list of contiguous rectangles
  * such that the last one has the smallest LL.y and its LL.y is above
  * the bottom of the rank (rank.ht1).
@@ -810,15 +806,15 @@ else
   return pair_a[tail_i][head_i];
 }
 
-static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
-                       double stepy, splineInfo *sinfo) {
+static void selfBottom(edge_t *edges[], size_t cnt, double sizex, double stepy,
+                       splineInfo *sinfo) {
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
     int sgn, point_pair;
     double hy, ty, stepx, dx, dy, height;
 
-    e = edges[ind];
+    e = *edges;
     n = agtail(e);
 
     stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
@@ -845,7 +841,7 @@ static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     ty = fmin(dy, 3 * (tp.y + dy - np.y));
     hy = fmin(dy, 3 * (hp.y + dy - np.y));
     for (size_t i = 0; i < cnt; i++) {
-        e = edges[ind++];
+        e = edges[i];
         dy += stepy;
         ty += stepy;
         hy += stepy;
@@ -880,15 +876,15 @@ static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     }
 }
 
-static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
-                    double stepy, splineInfo *sinfo) {
+static void selfTop(edge_t *edges[], size_t cnt, double sizex, double stepy,
+                    splineInfo *sinfo) {
     int sgn, point_pair;
     double hy, ty,  stepx, dx, dy, height;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
 
-    e = edges[ind];
+    e = *edges;
     n = agtail(e);
 
     stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
@@ -952,7 +948,7 @@ static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     ty = fmin(dy, 3 * (np.y + dy - tp.y));
     hy = fmin(dy, 3 * (np.y + dy - hp.y));
     for (size_t i = 0; i < cnt; i++) {
-        e = edges[ind++];
+        e = edges[i];
         dy += stepy;
         ty += stepy;
         hy += stepy;
@@ -987,15 +983,15 @@ static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     }
 }
 
-static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
-                      double sizey, splineInfo *sinfo) {
+static void selfRight(edge_t *edges[], size_t cnt, double stepx, double sizey,
+                      splineInfo *sinfo) {
     int sgn, point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
 
-    e = edges[ind];
+    e = *edges;
     n = agtail(e);
 
     stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
@@ -1023,7 +1019,7 @@ static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     tx = fmin(dx, 3 * (np.x + dx - tp.x));
     hx = fmin(dx, 3 * (np.x + dx - hp.x));
     for (size_t i = 0; i < cnt; i++) {
-        e = edges[ind++];
+        e = edges[i];
         dx += stepx;
         tx += stepx;
         hx += stepx;
@@ -1058,15 +1054,15 @@ static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     }
 }
 
-static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
-                     double sizey, splineInfo *sinfo) {
+static void selfLeft(edge_t *edges[], size_t cnt, double stepx, double sizey,
+                     splineInfo *sinfo) {
     int sgn,point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
 
-    e = edges[ind];
+    e = *edges;
     n = agtail(e);
 
     stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
@@ -1097,7 +1093,7 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     tx = fmin(dx, 3 * (tp.x + dx - np.x));
     hx = fmin(dx, 3 * (hp.x + dx - np.x));
     for (size_t i = 0; i < cnt; i++) {
-        e = edges[ind++];
+        e = edges[i];
         dx += stepx;
         tx += stepx;
         hx += stepx;
@@ -1133,8 +1129,7 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     }
 }
 
-/* selfRightSpace:
- * Assume e is self-edge.
+/* Assume e is self-edge.
  * Return extra space necessary on the right for this edge.
  * If the edge does not go on the right, return 0.
  * NOTE: the actual space is determined dynamically by GD_nodesep,
@@ -1161,15 +1156,14 @@ double selfRightSpace(edge_t *e) {
     return sw;
 }
 
-/* makeSelfEdge:
- * The routing is biased towards the right side because this is what
+/* The routing is biased towards the right side because this is what
  * dot supports, and leaves room for.
  * FIX: With this bias, labels tend to be placed on top of each other.
  * Perhaps for self-edges, the label should be centered.
  */
-void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
-                  double sizey, splineInfo * sinfo) {
-    edge_t *e = edges[ind];
+void makeSelfEdge(edge_t *edges[], size_t cnt, double sizex, double sizey,
+                  splineInfo *sinfo) {
+    edge_t *e = *edges;
 
     /* self edge without ports or
      * self edge with all ports inside, on the right, or at most 1 on top
@@ -1181,7 +1175,7 @@ void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
          !(ED_head_port(e).side & LEFT) &&
           (ED_tail_port(e).side != ED_head_port(e).side ||
           !(ED_tail_port(e).side & (TOP|BOTTOM))))) {
-	selfRight(edges, ind, cnt, sizex, sizey, sinfo);
+	selfRight(edges, cnt, sizex, sizey, sinfo);
     }
 
     /* self edge with port on left side */
@@ -1189,27 +1183,25 @@ void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
 
 	/* handle L-R specially */
 	if ((ED_tail_port(e).side & RIGHT) || (ED_head_port(e).side & RIGHT)) {
-	    selfTop(edges, ind, cnt, sizex, sizey, sinfo);
+	    selfTop(edges, cnt, sizex, sizey, sinfo);
 	}
 	else {
-	    selfLeft(edges, ind, cnt, sizex, sizey, sinfo);
+	    selfLeft(edges, cnt, sizex, sizey, sinfo);
 	}
     }
 
     /* self edge with both ports on top side */
     else if (ED_tail_port(e).side & TOP) {
-	selfTop(edges, ind, cnt, sizex, sizey, sinfo);
+	selfTop(edges, cnt, sizex, sizey, sinfo);
     }
     else if (ED_tail_port(e).side & BOTTOM) {
-	selfBottom(edges, ind, cnt, sizex, sizey, sinfo);
+	selfBottom(edges, cnt, sizex, sizey, sinfo);
     }
 
     else assert(0);
 }
 
-/* makePortLabels:
- * Add head and tail labels if necessary and update bounding box.
- */
+/// add head and tail labels if necessary and update bounding box
 void makePortLabels(edge_t * e)
 {
     /* Only use this if labelangle or labeldistance is set for the edge;
@@ -1227,10 +1219,7 @@ void makePortLabels(edge_t * e)
     }
 }
 
-/* endPoints:
- * Extract the actual end points of the spline, where
- * they touch the node.
- */
+/// extract the actual end points of the spline, where they touch the node
 static void endPoints(splines * spl, pointf * p, pointf * q)
 {
     bezier bz;
@@ -1251,8 +1240,7 @@ static void endPoints(splines * spl, pointf * p, pointf * q)
     }
 }
 
-/* polylineMidpoint;
- * Find midpoint of polyline.
+/* Find midpoint of polyline.
  * pp and pq are set to the endpoints of the line segment containing it.
  */
 static pointf
@@ -1295,15 +1283,14 @@ pointf
 edgeMidpoint (graph_t* g, edge_t * e)
 {
     int et = EDGE_TYPE (g);
-    pointf d, spf, p, q;
+    pointf spf, p, q;
 
     endPoints(ED_spl(e), &p, &q);
     if (APPROXEQPT(p, q, MILLIPOINT)) { /* degenerate spline */
 	spf = p;
     }
     else if (et == EDGETYPE_SPLINE || et == EDGETYPE_CURVED) {
-	d.x = (q.x + p.x) / 2.;
-	d.y = (p.y + q.y) / 2.;
+	const pointf d = mid_pointf(p, q);
 	spf = dotneato_closest(ED_spl(e), d);
     }
     else {   /* EDGETYPE_PLINE, EDGETYPE_ORTHO or EDGETYPE_LINE */
@@ -1313,11 +1300,7 @@ edgeMidpoint (graph_t* g, edge_t * e)
     return spf;
 }
 
-#define LEFTOF(a,b,c) (((a.y - b.y)*(c.x - b.x) - (c.y - b.y)*(a.x - b.x)) > 0)
-#define MAXLABELWD  (POINTS_PER_INCH/2.0)
-
-/* addEdgeLabels:
- * Adds label, headlabel and taillabel.
+/* Adds label, headlabel and taillabel.
  * Updates bounding box.
  * We use the endpoints of the spline.
  */
@@ -1325,60 +1308,52 @@ void addEdgeLabels(edge_t *e) {
     makePortLabels(e);
 }
 
-#define AGXGET(o,a) agxget(o,a)
-
-/* place_portlabel:
- * place the {head,tail}label (depending on HEAD_P) of edge E
+/* place the {head,tail}label (depending on HEAD_P) of edge E
  * N.B. Assume edges are normalized, so tail is at spl->list[0].list[0]
  * and head is at spl->list[spl->size-l].list[bez->size-1]
  * Return 1 on success
  */
 int place_portlabel(edge_t * e, bool head_p)
 {
-    textlabel_t *l;
     splines *spl;
-    bezier *bez;
-    double dist, angle;
-    pointf c[4], pe, pf;
-    char* la;
-    char* ld;
+    pointf pe, pf;
 
     if (ED_edge_type(e) == IGNORED)
 	return 0;
     /* add label here only if labelangle or labeldistance is defined; else, use external label */
-    if ((!E_labelangle || *(la = AGXGET(e,E_labelangle)) == '\0') &&
-	(!E_labeldistance || *(ld = AGXGET(e,E_labeldistance)) == '\0')) {
+    if ((!E_labelangle || streq(agxget(e, E_labelangle), "")) &&
+	(!E_labeldistance || streq(agxget(e, E_labeldistance), ""))) {
 	return 0;
     }
 
-    l = head_p ? ED_head_label(e) : ED_tail_label(e);
+    textlabel_t *const l = head_p ? ED_head_label(e) : ED_tail_label(e);
     if ((spl = getsplinepoints(e)) == NULL) return 0;
     if (!head_p) {
-	bez = &spl->list[0];
+	const bezier *const bez = &spl->list[0];
 	if (bez->sflag) {
 	    pe = bez->sp;
 	    pf = bez->list[0];
 	} else {
 	    pe = bez->list[0];
-	    for (size_t i = 0; i < 4; i++)
-		c[i] = bez->list[i];
+	    const pointf *const c = bez->list; // slice of the first 4 points
 	    pf = Bezier(c, 0.1, NULL, NULL);
 	}
     } else {
-	bez = &spl->list[spl->size - 1];
+	const bezier *const bez = &spl->list[spl->size - 1];
 	if (bez->eflag) {
 	    pe = bez->ep;
 	    pf = bez->list[bez->size - 1];
 	} else {
 	    pe = bez->list[bez->size - 1];
-	    for (size_t i = 0; i < 4; i++)
-		c[i] = bez->list[bez->size - 4 + i];
+	    // slice of the last 4 points
+	    const pointf *const c = &bez->list[bez->size - 4];
 	    pf = Bezier(c, 0.9, NULL, NULL);
 	}
     }
-    angle = atan2(pf.y - pe.y, pf.x - pe.x) +
+    const double angle = atan2(pf.y - pe.y, pf.x - pe.x) +
 	RADIANS(late_double(e, E_labelangle, PORT_LABEL_ANGLE, -180.0));
-    dist = PORT_LABEL_DISTANCE * late_double(e, E_labeldistance, 1.0, 0.0);
+    const double dist =
+      PORT_LABEL_DISTANCE * late_double(e, E_labeldistance, 1.0, 0.0);
     l->pos.x = pe.x + dist * cos(angle);
     l->pos.y = pe.y + dist * sin(angle);
     l->set = true;
