@@ -23,6 +23,8 @@
  *  watch out for interactions between leaves and clusters.
  */
 
+#include "config.h"
+
 #include	<dotgen/dot.h>
 #include	<limits.h>
 #include	<stdbool.h>
@@ -35,14 +37,14 @@
 static void dot1_rank(graph_t *g);
 static void dot2_rank(graph_t *g);
 
-DEFINE_LIST(edge_set, edge_t *)
+typedef LIST(edge_t *) edge_set_t;
 
 /// @param track An optional collection in which to record non-null entries we
 ///   removed
 static void renewlist(elist *L, edge_set_t *track) {
     for (size_t i = L->size; i != SIZE_MAX; i--) {
 	if (track != NULL && L->list[i] != NULL) {
-            edge_set_append(track, L->list[i]);
+            LIST_APPEND(track, L->list[i]);
 	}
 	L->list[i] = NULL;
     }
@@ -60,7 +62,9 @@ static void renewlist(elist *L, edge_set_t *track) {
 /// @param a One edge
 /// @param b Another edge
 /// @return A comparator value suitable for sorting algorithms
-static int edge_ptr_cmp(const edge_t **a, const edge_t **b) {
+static int edge_ptr_cmp(const void *x, const void *y) {
+  const edge_t *const *a = x;
+  const edge_t *const *b = y;
   const uintptr_t addr_a = (uintptr_t)*a;
   const uintptr_t addr_b = (uintptr_t)*b;
   if (addr_a < addr_b) {
@@ -75,21 +79,41 @@ static int edge_ptr_cmp(const edge_t **a, const edge_t **b) {
 static void 
 cleanup1(graph_t * g)
 {
-    node_t *n;
     edge_t *e, *f;
 
     edge_set_t to_free = {0};
 
     for (size_t c = 0; c < GD_comp(g).size; c++) {
 	    GD_nlist(g) = GD_comp(g).list[c];
-	    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	    for (node_t *n = GD_nlist(g), *next, *prev = NULL; n; n = next) {
+	        next = ND_next(n);
 	        // out edges are owning, so only track their removal
 	        renewlist(&ND_in(n), NULL);
 	        renewlist(&ND_out(n), &to_free);
 	        ND_mark(n) = false;
+	        // If this is a slack node, it exists _only_ in the component lists
+	        // that we are about to drop. Remove and deallocate slack nodes now to
+	        // avoid leaking these.
+	        if (ND_node_type(n) == SLACKNODE) {
+                if (prev == NULL) {
+                    GD_comp(g).list[c] = next;
+                    GD_nlist(g) = next;
+                } else {
+                    ND_next(prev) = next;
+                }
+                if (next != NULL) {
+                    ND_prev(next) = prev;
+                }
+                free_list(ND_in(n));
+                free_list(ND_out(n));
+                free(n->base.data);
+                free(n);
+	        } else {
+                prev = n;
+	        }
 	    }
     }
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (node_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
         for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
             f = ED_to_virt(e);
             /* Null out any other references to f to make sure we don't
@@ -101,11 +125,11 @@ cleanup1(graph_t * g)
             }
         }
     }
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (node_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
         for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
             f = ED_to_virt(e);
             if (f && ED_to_orig(f) == e) {
-                edge_set_append(&to_free, f);
+                LIST_APPEND(&to_free, f);
                 ED_to_virt(e) = NULL;
             }
 	    }
@@ -115,17 +139,17 @@ cleanup1(graph_t * g)
     // XXX: Accruing all the pointers, including duplicates, and then sorting to
     // avoid duplicate frees is suboptimal. If this turns out to be a
     // performance problem, replace `edge_set_t` with a proper set.
-    edge_set_sort(&to_free, edge_ptr_cmp);
+    LIST_SORT(&to_free, edge_ptr_cmp);
     edge_t *previous = NULL;
-    for (size_t i = 0; i < edge_set_size(&to_free); ++i) {
-        edge_t *const current = edge_set_get(&to_free, i);
+    for (size_t i = 0; i < LIST_SIZE(&to_free); ++i) {
+        edge_t *const current = LIST_GET(&to_free, i);
         if (current != previous) {
             free(current->base.data);
             free(current);
         }
         previous = current;
     }
-    edge_set_free(&to_free);
+    LIST_FREE(&to_free);
 
     free(GD_comp(g).list);
     GD_comp(g).list = NULL;
@@ -327,7 +351,7 @@ collapse_sets(graph_t *rg, graph_t *g)
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 	c = rank_set_class(subg);
 	if (c) {
-	    if ((c == CLUSTER) && CL_type == LOCAL)
+	    if (c == CLUSTER && CL_type == LOCAL)
 		collapse_cluster(rg, subg);
 	    else
 		collapse_rankset(rg, subg, c);
@@ -369,7 +393,7 @@ minmax_edges(graph_t * g)
     point  slen;
 
     slen.x = slen.y = 0;
-    if ((GD_maxset(g) == NULL) && (GD_minset(g) == NULL))
+    if (GD_maxset(g) == NULL && GD_minset(g) == NULL)
 	return slen;
     if (GD_minset(g) != NULL)
 	GD_minset(g) = UF_find(GD_minset(g));
@@ -377,14 +401,14 @@ minmax_edges(graph_t * g)
 	GD_maxset(g) = UF_find(GD_maxset(g));
 
     if ((n = GD_maxset(g))) {
-	slen.y = (ND_ranktype(GD_maxset(g)) == SINKRANK);
+	slen.y = ND_ranktype(GD_maxset(g)) == SINKRANK;
 	while ((e = ND_out(n).list[0])) {
 	    assert(aghead(e) == UF_find(aghead(e)));
 	    reverse_edge(e);
 	}
     }
     if ((n = GD_minset(g))) {
-	slen.x = (ND_ranktype(GD_minset(g)) == SOURCERANK);
+	slen.x = ND_ranktype(GD_minset(g)) == SOURCERANK;
 	while ((e = ND_in(n).list[0])) {
 	    assert(agtail(e) == UF_find(agtail(e)));
 	    reverse_edge(e);
@@ -399,23 +423,23 @@ minmax_edges2(graph_t * g, point slen)
     node_t *n;
     edge_t *e = 0;
 
-    if ((GD_maxset(g)) || (GD_minset(g))) {
+    if (GD_maxset(g) || GD_minset(g)) {
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    if (n != UF_find(n))
 		continue;
-	    if ((ND_out(n).size == 0) && GD_maxset(g) && (n != GD_maxset(g))) {
+	    if (ND_out(n).size == 0 && GD_maxset(g) && n != GD_maxset(g)) {
 		e = virtual_edge(n, GD_maxset(g), NULL);
 		ED_minlen(e) = slen.y;
 		ED_weight(e) = 0;
 	    }
-	    if ((ND_in(n).size == 0) && GD_minset(g) && (n != GD_minset(g))) {
+	    if (ND_in(n).size == 0 && GD_minset(g) && n != GD_minset(g)) {
 		e = virtual_edge(GD_minset(g), n, NULL);
 		ED_minlen(e) = slen.x;
 		ED_weight(e) = 0;
 	    }
 	}
     }
-    return (e != 0);
+    return e != 0;
 }
 
 /* Run the network simplex algorithm on each component. */
@@ -428,7 +452,7 @@ void rank1(graph_t * g)
 	maxiter = scale_clamp(agnnodes(g), atof(s));
     for (size_t c = 0; c < GD_comp(g).size; c++) {
 	GD_nlist(g) = GD_comp(g).list[c];
-	rank(g, (GD_n_cluster(g) == 0 ? 1 : 0), maxiter);	/* TB balance */
+	rank(g, GD_n_cluster(g) == 0 ? 1 : 0, maxiter); // TB balance
     }
 }
 
@@ -765,7 +789,7 @@ static void weak(graph_t * g, node_t * t, node_t * h, edge_t * orig)
     for (e = agfstin(g, t); e; e = agnxtin(g, e)) {
 	/* merge with existing weak edge (e,f) */
 	v = agtail(e);
-	if ((f = agfstout(g, v)) && (aghead(f) == h)) {
+	if ((f = agfstout(g, v)) && aghead(f) == h) {
 	    return;
 	}
     }
@@ -807,9 +831,7 @@ static void compile_edges(graph_t * ug, graph_t * Xg)
 		/* determine if graph requires reversed edge */
 		if ((clust_tail != NULL && find(agtail(e)) == GD_maxrep(clust_tail))
 		    || (clust_head != NULL && find(aghead(e)) == GD_minrep(clust_head))) {
-		    node_t *temp = Xt;
-		    Xt = Xh;
-		    Xh = temp;
+		    SWAP(&Xt, &Xh);
 		}
 		strong(Xg, Xt, Xh, e);
 	    } else {
