@@ -12,6 +12,8 @@
  *  Code for main functions in gpr
  */
 
+#include "config.h"
+
 #include <ast/ast.h>
 #include <ast/error.h>
 #include <cgraph/agstrcanon.h>
@@ -281,14 +283,14 @@ static void cloneGraph(Agraph_t *tgt, Agraph_t *src) {
   edgepair_t *ep = data;
 
   for (t = agfstnode(src); t; t = agnxtnode(src, t)) {
-    if (!copy(tgt, OBJ(t))) {
+    if (!copy(tgt, &t->base)) {
       exerror("error cloning node %s from graph %s", agnameof(t),
               agnameof(src));
     }
   }
   for (t = agfstnode(src); t; t = agnxtnode(src, t)) {
     for (e = agfstout(src, t); e; e = agnxtout(src, e)) {
-      if (!(ne = (Agedge_t *)copy(tgt, OBJ(e)))) {
+      if (!(ne = (Agedge_t *)copy(tgt, &e->base))) {
         name = agnameof(AGMKOUT(e));
         if (name)
           exerror("error cloning edge (%s,%s)[%s] from graph %s",
@@ -325,7 +327,7 @@ Agraph_t *cloneG(Agraph_t *g, char *name) {
     name = agnameof(g);
   ng = openG(name, g->desc);
   if (ng) {
-    copyAttr((Agobj_t *)g, (Agobj_t *)ng);
+    copyAttr(&g->base, &ng->base);
     cloneGraph(ng, g);
   }
   return ng;
@@ -491,7 +493,7 @@ int lockGraph(Agraph_t *g, int v) {
     if (data->lock.zombie)
       agclose(g);
     else
-      data->lock = (lock_t){0};
+      data->lock = (gvlock_t){0};
   }
   return oldv;
 }
