@@ -14,6 +14,8 @@
  * Written by Emden Gansner
  */
 
+#include "config.h"
+
 #include "builddate.h"
 #include <assert.h>
 #include <ast/error.h>
@@ -30,6 +32,7 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
@@ -75,6 +78,13 @@ typedef struct {
   int verbose;
 } options;
 
+static clock_t start_timer(void) { return clock(); }
+
+static double elapsed_sec(clock_t start) {
+  const clock_t end = clock();
+  return (end - start) / (double)CLOCKS_PER_SEC;
+}
+
 static FILE *openOut(char *name) {
   FILE *const outs = gv_fopen(name, "w");
   if (outs == 0) {
@@ -829,7 +839,7 @@ static void addOutputGraph(Gpr_t *state, gvpropts *uopts) {
   Agraph_t *g = state->outgraph;
 
   if ((agroot(g) == state->curgraph) && !uopts->ingraphs)
-    g = (Agraph_t *)cloneO(0, (Agobj_t *)g);
+    g = (Agraph_t *)cloneO(0, &g->base);
 
   uopts->outgraphs = gv_recalloc(uopts->outgraphs, uopts->n_outgraphs,
                                  uopts->n_outgraphs + 1, sizeof(Agraph_t *));
@@ -914,8 +924,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     return gs->opts.state;
   }
 
-  if (gs->opts.verbose)
-    gvstart_timer();
+  clock_t start = start_timer();
   gs->prog = parseProg(gs->opts.program, gs->opts.useFile);
   if (gs->prog == NULL) {
     return 1;
@@ -956,7 +965,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
   bool incoreGraphs = uopts->ingraphs;
 
   if (gs->opts.verbose)
-    fprintf(stderr, "Parse/compile/init: %.2f secs.\n", gvelapsed_sec());
+    fprintf(stderr, "Parse/compile/init: %.2f secs.\n", elapsed_sec(start));
   /* do begin */
   if (gs->xprog->begin_stmt != NULL)
     exeval(gs->xprog->prog, gs->xprog->begin_stmt, gs->state);
@@ -968,13 +977,12 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     else
       gs->ing = newIng(0, gs->opts.inFiles, ing_read);
 
-    if (gs->opts.verbose)
-      gvstart_timer();
+    start = start_timer();
     Agraph_t *nextg = NULL;
     for (gs->state->curgraph = nextGraph(gs->ing); gs->state->curgraph;
          gs->state->curgraph = nextg) {
       if (gs->opts.verbose)
-        fprintf(stderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Read graph: %.2f secs.\n", elapsed_sec(start));
       gs->state->infname = fileName(gs->ing);
       if (gs->opts.readAhead)
         nextg = gs->state->nextgraph = nextGraph(gs->ing);
@@ -1003,7 +1011,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
       if (gs->xprog->endg_stmt != NULL)
         exeval(gs->xprog->prog, gs->xprog->endg_stmt, gs->state);
       if (gs->opts.verbose)
-        fprintf(stderr, "Finish graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Finish graph: %.2f secs.\n", elapsed_sec(start));
 
       /* if $O == $G and $T is empty, delete $T */
       if (gs->state->outgraph == gs->state->curgraph &&
@@ -1027,12 +1035,11 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
       gs->state->target = 0;
       gs->state->outgraph = 0;
 
-      if (gs->opts.verbose)
-        gvstart_timer();
+      start = start_timer();
       if (!gs->opts.readAhead)
         nextg = nextGraph(gs->ing);
       if (gs->opts.verbose && nextg != NULL) {
-        fprintf(stderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Read graph: %.2f secs.\n", elapsed_sec(start));
       }
     }
   }
